"""Application layer orchestrators (ports + merge policy).

Purpose
-------
Define pure coordination code that glues domain value objects to adapter
interfaces.

Contents
--------
* :mod:`lib_layered_config.application.ports`
* :mod:`lib_layered_config.application.merge`
"""
